'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _dropbox = require('dropbox');

var _dropbox2 = _interopRequireDefault(_dropbox);

var _dropboxStream = require('dropbox-stream');

var _dropboxStream2 = _interopRequireDefault(_dropboxStream);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const TYPE_KEY = '@@fsType';
/**
 * Create an fs-like API for Dropbox
 *
 * @param {{
 *  apiKey: String,
 *  client: Dropbox
 * }} Configuration object
 * @returns {Object}
 */

exports.default = ({ apiKey = null, client = null } = {}) => {
    if (!client && typeof apiKey === 'string') {
        client = new _dropbox2.default({
            accessToken: apiKey
        });
    } else if (!client) {
        throw new Error('Dropbox client or apiKey should be provided.');
    }

    const fs = require('./index')({ apiKey, client });

    const api = {
        [TYPE_KEY]: 'dropbox-fs',

        /**
         * Read a directory and list all the files and folders inside
         *
         * @param {String} remotePath
         * @param {Object} options
         * @return {Promise<(DropboxTypes.files.FileMetadataReference | DropboxTypes.files.FolderMetadataReference | DropboxTypes.files.DeletedMetadataReference)[]>}
         */
        readdir(remotePath = '', options = {}) {
            return new Promise((resolve, reject) => {
                fs.readdir(remotePath, options, (err, entries) => {
                    if (err) {
                        return reject(err);
                    }
                    resolve(entries);
                });
            });
        },

        /**
         * Create a remote directory
         *
         * @param {String} remotePath
         * @return {Promise<DropboxTypes.files.FolderMetadata>}
         */
        mkdir(remotePath) {
            return new Promise((resolve, reject) => {
                fs.mkdir(remotePath, (err, metadata) => {
                    if (err) {
                        return reject(err);
                    }
                    resolve(metadata);
                });
            });
        },

        /**
         * Read a remote file and return it’s contents
         *
         * @param {String} remotePath
         * @param {Object} options
         * @return {Promise<Buffer>}
         */
        readFile(remotePath, options = { encoding: null }) {
            return _asyncToGenerator(function* () {
                return new Promise(function (resolve, reject) {
                    fs.readFile(remotePath, options, function (err, buffer) {
                        if (err) {
                            return reject(err);
                        }
                        resolve(buffer);
                    });
                });
            })();
        },

        /**
         * Rename (move) a remote file
         *
         * @param {String} fromPath
         * @param {String} toPath
         * @return {Promise<void>}
         */
        rename(fromPath, toPath) {
            return new Promise((resolve, reject) => {
                fs.rename(fromPath, toPath, err => {
                    if (err) {
                        return reject(err);
                    }
                    resolve();
                });
            });
        },

        /**
         * Return file or folder meta data
         *
         * @param {String} remotePath
         * @return {Promise<DropboxTypes.files.FileMetadataReference | DropboxTypes.files.FolderMetadataReference | DropboxTypes.files.DeletedMetadataReference>}
         */
        stat(remotePath) {
            return new Promise((resolve, reject) => {
                fs.stat(remotePath, (err, meta) => {
                    if (err) {
                        return reject(err);
                    }
                    resolve(meta);
                });
            });
        },

        /**
         * Delete a file or folder
         *
         * @param {String} remotePath
         * @return {Promise<void>}
         */
        unlink(remotePath) {
            return new Promise((resolve, reject) => {
                fs.unlink(remotePath, err => {
                    if (err) {
                        return reject(err);
                    }
                    resolve();
                });
            });
        },

        /**
         * create write stream
         *
         * @param {String} token
         * @param {String} remotePath
         * @returns {Stream}
         */
        createWriteStream(filepath) {
            return _dropboxStream2.default.createDropboxUploadStream({
                token: client.accessToken,
                filepath,
                chunkSize: 1000 * 1024
            });
        },

        /**
         * create read stream
         *
         * @param {String} token
         * @param {String} remotePath
         * @returns {Stream}
         */
        createReadStream(filepath) {
            return _dropboxStream2.default.createDropboxDownloadStream({
                token: client.accessToken,
                filepath
            });
        },

        /**
         * Write a file
         *
         * @param {String} remotePath
         * @param {String|Buffer} data
         * @param {Object|String} options
         * @return {Promise<DropboxTypes.files.FileMetadata>}
         */
        writeFile(remotePath, data, options = {}) {
            return new Promise((resolve, reject) => {
                fs.writeFile(remotePath, data, options, (err, meta) => {
                    if (err) {
                        return reject(err);
                    }
                    resolve(meta);
                });
            });
        }
    };

    api.rmdir = api.unlink;

    return api;
};

module.exports = exports['default'];